<?php
namespace ServerlessFC;

use Monolog\Formatter\LineFormatter;
use Monolog\Handler\RotatingFileHandler;
use Monolog\Handler\StreamHandler;
use Monolog\Logger;

require dirname(__DIR__) . '/builtIn/vendor/autoload.php';

$fcLogger      = null;
$fcSysLogger   = null;
$fcPhpCgiProxy = null;

class FcLineFormatter extends LineFormatter {
    public function format(array $record) {
        $vars   = parent::format($record);
        $output = $this->format;
        $output = str_replace('[]', $GLOBALS['requestId'], $vars);
        return $output;
    }
}

class FcLogger extends Logger {
    public function setLevel($level) {
        $handlers = $this->getHandlers();
        foreach ($handlers as &$h) {
            $h->setLevel($level);
        }
    }
}

function init_logging($out = 'php://output') {
    // for users
    $output        = "%datetime% %context% [%level_name%]: %message%\n";
    $formatter     = new FcLineFormatter($output, "Y-m-d\TH:i:s\Z");
    $streamHandler = new StreamHandler($out, Logger::INFO);
    $streamHandler->setFormatter($formatter);
    // add setLevel method in logger/vendor/momolog/src/Logger.php
    global $fcLogger;
    $fcLogger = new FcLogger('fcLogger');
    $fcLogger->pushHandler($streamHandler);

    $caLogFile        = sprintf("%s/cagent_php.log", getenv(FC_SERVER_LOG_PATH));
    $sysOutput        = "%datetime% [%level_name%] %message%\n";
    $sysFormatter     = new LineFormatter($sysOutput, "Y-m-d H:i:s.u");
    $sysStreamHandler = new RotatingFileHandler($caLogFile, 0, Logger::INFO);
    $sysStreamHandler->setFormatter($sysFormatter);
    global $fcSysLogger;
    $fcSysLogger = new Logger('fcSysLogger');
    $fcSysLogger->pushHandler($sysStreamHandler);
}

function sanitize_envs() {
    foreach ($_ENV as $key => $val) {
        if (startsWith($key, "FC_") and (!in_array($key, FC_SAFE_ENV))) {
            unset($_ENV[$key]);
        }
    }
}

function init_fc_cgi_proxy() {
    global $fcPhpCgiProxy;
    $fcPhpCgiProxy = new PhpCgiProxy();
}
