<?php
namespace ServerlessFC;

require_once "autoload.php";

use Psr\Http\Message\ServerRequestInterface;
use React\EventLoop\Factory;
use React\Http\Response;
use React\Http\Server;

function gen_response($content, $status_code) {
    return new Response(
        $status_code,
        array(
            'Content-Type'   => 'application/octet-stream',
            'Content-Length' => strlen($content),
            'Connection'     => 'keep-alive',
        ),
        $content
    );
}

$loop = Factory::create();
$path_name = array("/initialize", "/invoke");

$server = new Server(function (ServerRequestInterface $request) {
    global $path_name;
    $headers       = array_change_key_case($request->getHeaders(), CASE_LOWER);
    $requestId     = array_key_exists(FC_REQUEST_ID, $headers) ? $headers[FC_REQUEST_ID][0] : "unknown_request_id";
    $req_path_name = $request->getUri()->getPath();

    if (ob_get_length()) {
        ob_clean();
    }

    echo FC_LOG_TAIL_START_PREFIX . $requestId . PHP_EOL;
    global $fcSysLogger;
    $fcSysLogger->info(FC_LOG_TAIL_START_PREFIX . $requestId);

    $GLOBALS['requestId'] = $requestId;

    try {
        $errTxt = (in_array($req_path_name, $path_name)) ? '' : 'request path must be /invoke or /initialize';
        if ($errTxt) {
            return gen_response($errTxt, 404);
        }
        $context     = gen_fc_context($headers);
        $entry_point = null;
        if ($req_path_name == $path_name[0]) {
            $entry_point = array_key_exists(FC_FUNCTION_INITIALIZER, $headers) ? $headers[FC_FUNCTION_INITIALIZER][0] : '';
        } else {
            $entry_point = array_key_exists(FC_FUNCTION_HANDLER, $headers) ? $headers[FC_FUNCTION_HANDLER][0] : '';
        }

        $retArr = get_module_path($entry_point);
        if ($retArr[0]) {
            return gen_response($retArr[0], 404);
        }

        $http_params = $request->getHeader(FC_HTTP_PARAMS);
        if ($http_params) {
            $httpHandler = new HttpInvokeHandler($retArr[1], $retArr[2]);
            $response    = $httpHandler->handle($request, $context);
            return $response;
        } else {
            $ret = null;
            if ($req_path_name == $path_name[0]) {
                // for intialize request
                $normalHandler = new NormalCARequest($retArr[1], $retArr[2]);
                $ret           = $normalHandler->initialize($context);

            } else {
                // for invoke request
                $event         = $request->getBody()->getContents();
                $normalHandler = new NormalCARequest($retArr[1], $retArr[2]);
                $ret           = $normalHandler->invoke($event, $context);
            }
            return gen_response($ret, 200);
        }
    } catch (Throwable $e) {
        // all can be catched throwable(error and exception) is processed in http/src/StreamingServer fcWriteError
    }

});

sanitize_envs();
init_logging();
init_fc_cgi_proxy();

echo 'FunctionCompute php7.2 runtime inited.' . PHP_EOL;

$port = $_ENV[FC_SERVER_PORT] ?: '9000';

$socket = new \React\Socket\Server(sprintf('0.0.0.0:%s', $port), $loop);
$server->listen($socket);

$loop->run();
